/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.plugins.sisu.assembler;

import java.io.File;
import java.io.FileWriter;
import java.net.URI;
import java.util.Arrays;
import java.util.Collection;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.assembly.model.Component;
import org.apache.maven.plugin.assembly.model.FileItem;
import org.apache.maven.plugin.assembly.model.FileSet;
import org.apache.maven.plugin.assembly.model.io.xpp3.ComponentXpp3Writer;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.IOUtil;
import org.sonatype.p2.touchpoint.mixin.generic.internal.PermissionRegistry;
import org.sonatype.p2.touchpoint.mixin.generic.internal.PermissionRegistry.Entry;

/**
 * @goal generate-assembly-xml
 * @phase package
 */
public class GenerateAssemblyXmlMojo
    extends AbstractMojo
{

    /**
     * @parameter expression="${project.build.directory}/p2/p2-install-folder"
     * @required
     */
    private File installDirectory;

    /**
     * @parameter default-value="/"
     */
    private String outputDirectory;

    /**
     * @parameter expression="${project.build.directory}/p2/assembly-descriptor.xml"
     * @required
     */
    private File assemblyXml;

    /**
     * @parameter expression="${project.build.directory}"
     * @readonly
     */
    private File projectBuildDirectory;

    /**
     * @parameter expression="${project}"
     * @readonly
     */
    protected MavenProject project;

    /**
     * {@inheritDoc}
     */
    public void execute()
        throws MojoExecutionException
    {
        final URI projectDirUri = projectBuildDirectory.getParentFile().toURI();
        final URI assembleXmlUri = assemblyXml.toURI();
        final URI relativeUri = projectDirUri.relativize( assembleXmlUri );
        String generatedAssemblyXml = assemblyXml.toURI().toASCIIString();
        if ( !relativeUri.equals( assembleXmlUri ) )
        {
            generatedAssemblyXml = relativeUri.toASCIIString();
        }
        project.getProperties().setProperty( "sisu.assembler.generatedAssemblyXml", generatedAssemblyXml );

        final Component component = new Component();
        final FileSet fileSet = new FileSet();
        fileSet.setDirectory( installDirectory.getAbsolutePath() );
        fileSet.setExcludes( Arrays.asList( "p2/org.eclipse.equinox.p2.core/cache/" ) );
        fileSet.setOutputDirectory( outputDirectory );

        component.addFileSet( fileSet );

        final File dataArea = new File( installDirectory, "p2/org.sonatype.p2.touchpoint" );
        final File profileDirectory = new File( dataArea, "master.profile" );
        final File permissionFile = new File( profileDirectory, "permissions" );
        if ( permissionFile.exists() )
        {
            final PermissionRegistry permissionRegistry = new PermissionRegistry( permissionFile );
            final Collection<Entry> permissions = permissionRegistry.getPermissions();
            for ( final Entry entry : permissions )
            {
                final FileItem fileItem = new FileItem();

                fileItem.setSource( new File( installDirectory, entry.getTarget().getPath() ).getAbsolutePath() );
                fileItem.setDestName( entry.getTarget().getPath() );
                fileItem.setFileMode( entry.getPermission() );
                fileItem.setOutputDirectory( outputDirectory );

                component.addFile( fileItem );
            }
        }

        FileWriter writer = null;
        try
        {
            writer = new FileWriter( assemblyXml );
            new ComponentXpp3Writer().write( writer, component );
        }
        catch ( final Exception e )
        {
            IOUtil.close( writer );
        }
    }
}
